__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import os

from aerialer.Commander import Commander
from aerialer.cluster.Slurm import Slurm
from aerialer.Generator import Generator
from aerialer.BashCLI import BashCLI


class Transmitter:

    def __init__(
            self,
            config,
            job_fp,
            job_fn,
            scheduler,
            commander,
            cluster='slurm',
            method='slurm_to_many',
            job_suffix='.job',
            **kwargs,
    ):
        self.config = config
        self.job_fp = job_fp
        self.job_fn = job_fn
        self.job_suffix = job_suffix
        self.kwargs = kwargs
        self.method = method
        self.cluster = cluster
        self.scheduler = scheduler
        self.commander = commander

        self.kwargs['config'] = self.config
        self.kwargs['job_fn'] = self.job_fn
        self.kwargs['job_fp'] = self.job_fp
        self.kwargs['job_suffix'] = self.job_suffix

        self.kwargs['cmd_line'] = []

        ### /*** set cluster ***/
        if self.cluster == 'slurm':
            ### /*** generate headlines ***/
            self.Slurm = Slurm(**self.kwargs)
            self.scheduler_menu = {
                "sbatch": self.Slurm.sbatch,
                "sbatch_customised": self.Slurm.sbatch_customised,
            }
            self.scheduler_menu[self.scheduler]()

        ### /*** generate commands ***/
        self.Commander = Commander(**self.kwargs)
        self.command_tool_menu = {
            "dict": self.Commander.tool_py_dict,
        }
        self.Commander.pre()
        self.command = self.command_tool_menu[self.commander]()
        print("======>The CMD (software): {}".format(self.command))
        self.Commander.post()

        ### /*** generate job files ***/
        self.generator = Generator(**self.kwargs)
        self.generator.transcribe()

        self.bashcli = BashCLI(**self.kwargs)
        self.command_shell_menu = {
            "slurm_to_many": self.bashcli.shell_slurm_to_many,
        }
        self.cmd_in_shell = self.command_shell_menu[self.method]()
        print("======>The CMD (shell): {}".format(self.cmd_in_shell))

    def run(self, ):
        os.system(self.cmd_in_shell)
        return 'Job {} submitted!'.format(self.job_fn)


if __name__ == "__main__":
    import pandas as pd

    df = pd.read_csv(
        'D:\Document\Programming\Python\minverse\minverse\data\deepisite/cplx.txt',
        sep='\t',
        header=None,
    )
    print(df)
    prots = df[0].unique()[:2]
    print(prots)

    config = {
        "slurm": [
            "#!/usr/bin/env bash",
            "#SBATCH --job-name=myjob",
            "#SBATCH --nodes=1",
            "#SBATCH --time=23:00:00",
            "#SBATCH --ntasks=1",
            "#SBATCH --cpus-per-task=2",
            "#SBATCH --mem=8G",
            "#SBATCH --partition=short",
        ],
        "pre": [
            "source ~/.bashrc",
            "conda activate pbmc",
        ],
        "cmd" : {
            'pypropel_struct_check_cplx': '',
            '-fp': '/project/cribbslab/shared/proj048/sun/pdbtm/',
            '-fn': '',
            '-o': '/project/cribbslab/shared/proj048/sun/check/',
            '-t': 5.5,
            '-vb': False,
        }
    }


    for key, prot in enumerate(prots):
        config['cmd']['-fn'] = prot
        p = Transmitter(
            config=config,
            job_fp='./data/',
            job_fn=str(key),
            cluster='slurm',
            scheduler='sbatch_customised',
            commander='dict',
        )
        # p.run()


__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

import os


class Generator:

    def __init__(
            self,
            **kwargs,
    ):
        self.kwargs = kwargs
        if not os.path.exists(self.kwargs['job_fp']):
            os.makedirs(self.kwargs['job_fp'])
        self.job_fpn = self.kwargs['job_fp'] + self.kwargs['job_fn'] + '.job'

    def transcribe(self, ):
        with open(self.job_fpn, 'w') as job:
            job.write("\n".join(self.kwargs['cmd_line']) + "\n")


__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class Slurm:

    def __init__(
            self,
            **kwargs,
    ):
        self.kwargs = kwargs
        # print('Slurm: ', self.kwargs)
        if 'node' in self.kwargs.keys():
            self.node = self.kwargs['node']
        else:
            self.node = "1"
        if 'time' in self.kwargs.keys():
            self.time = self.kwargs['time']
        else:
            self.time="23:00:00"
        if 'num_tasks' in self.kwargs.keys():
            self.num_tasks = self.kwargs['num_tasks']
        else:
            self.num_tasks = "1"
        if 'cpu' in self.kwargs.keys():
            self.cpu = self.kwargs['cpu']
        else:
            self.cpu = "2"
        if 'memory' in self.kwargs.keys():
            self.memory = self.kwargs['memory']
        else:
            self.memory = "8"
        if 'partition' in self.kwargs.keys():
            self.partition = self.kwargs['partition']
        else:
            self.partition = 'short'
        if 'job_fn' in self.kwargs.keys():
            self.job_fn = self.kwargs['job_fn']
        if 'job_fpn' in self.kwargs.keys():
            self.job_fpn = self.kwargs['job_fpn']
        if 'command' in self.kwargs.keys():
            self.command = self.kwargs['command']
        if 'slurm' in self.kwargs.keys():
            self.slurm = self.kwargs['slurm']

    def sbatch(self, ):
        with open(self.job_fpn, 'w') as job:
            # job.writelines("#!/bin/bash\n")
            job.writelines("#!/usr/bin/env bash\n")
            job.writelines("#SBATCH --job-name={}\n".format(self.job_fn))
            job.writelines("#SBATCH --nodes={}\n".format(self.node))
            job.writelines("#SBATCH --time={}\n".format(self.time))
            job.writelines("#SBATCH --ntasks={}\n".format(self.num_tasks))
            job.writelines("#SBATCH --cpus-per-task={}\n".format(str(self.cpu)))
            job.writelines("#SBATCH --mem={}G\n".format(str(self.memory)))
            job.writelines("#SBATCH --partition={}\n".format(str(self.partition)))
            job.writelines("\n")
            job.writelines(self.command+"\n")
        return "sbatch {}".format(self.job_fpn)

    def sbatch_customised(self, ):
        lines = self.kwargs['config'].get("slurm", [])

        self.kwargs['cmd_line'].extend(lines)

        # Blank line between this and the rest
        if self.kwargs['cmd_line']:
            self.kwargs['cmd_line'].append("")



__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"

from aerialer.Transmitter import Transmitter


def submit(
        config,
        job_fp,
        job_fn,
        scheduler,
        commander,
        cluster='slurm',
        method='slurm_to_many',
        job_suffix='.job',
) -> str:
    return Transmitter(
        config=config,
        job_fp=job_fp,
        job_fn=job_fn,
        scheduler=scheduler,
        commander=commander,
        cluster=cluster,
        method=method,
        job_suffix=job_suffix,
    ).run()


if __name__ == "__main__":
    import pandas as pd

    df = pd.read_csv(
        'D:\Document\Programming\Python\minverse\minverse\data\deepisite/cplx.txt',
        sep='\t',
        header=None,
    )
    print(df)
    prots = df[0].unique()[:2]
    print(prots)

    config = {
        "slurm": [
            "#!/usr/bin/env bash",
            "#SBATCH --job-name=myjob",
            "#SBATCH --nodes=1",
            "#SBATCH --time=23:00:00",
            "#SBATCH --ntasks=1",
            "#SBATCH --cpus-per-task=2",
            "#SBATCH --mem=8G",
            "#SBATCH --partition=short",
        ],
        "pre": [
            "source ~/.bashrc",
            "conda activate pbmc",
        ],
        "cmd": {
            'pypropel_struct_check_cplx': '',
            '-fp': '/project/cribbslab/shared/proj048/sun/pdbtm/',
            '-fn': '',
            '-o': '/project/cribbslab/shared/proj048/sun/check/',
            '-t': 5.5,
            '-vb': False,
        }
    }

    for key, prot in enumerate(prots):
        config['cmd']['-fn'] = prot
        submit(
            config=config,
            job_fp='./data/',
            job_fn=str(prot),
            cluster='slurm',
            scheduler='sbatch_customised',
            commander='dict',
            method='slurm_to_many',
        )


__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class Commander:

    def __init__(
            self,
            **kwargs,
    ):
        self.kwargs = kwargs

    def tool_py_list(self, ):
        parts = []
        for i in range(len(self.cmd_config)):
            parts.append(self.cmd_config[i])
        cmd = " ".join(parts)
        # print(cmd)
        return cmd

    def tool_py_dict(self, ):
        cc = self.kwargs['config'].get("cmd", {})
        parts = []
        for k, v in cc.items():
            if v == "":
                parts.append(k)
            else:
                parts.extend([k, str(v)])
        cmd = " ".join(parts)
        # print(cmd)
        line = [cmd]
        self.kwargs['cmd_line'].extend(line)
        # Blank line between this and the rest
        if self.kwargs['cmd_line']:
            self.kwargs['cmd_line'].append("")
        return cmd

    def pre(self, ):
        if 'pre' in self.kwargs['config'].keys():
            lines = self.kwargs['config'].get("pre", [])

            self.kwargs['cmd_line'].extend(lines)

            # Blank line between this and the rest
            if self.kwargs['cmd_line']:
                self.kwargs['cmd_line'].append("")

    def post(self, ):
        if 'post' in self.kwargs['config'].keys():
            lines = self.kwargs['config'].get("post", [])

            self.kwargs['cmd_line'].extend(lines)

            # Blank line between this and the rest
            if self.kwargs['cmd_line']:
                self.kwargs['cmd_line'].append("")


__author__ = "Jianfeng Sun"
__version__ = "v1.0"
__copyright__ = "Copyright 2025"
__license__ = "GPL v3.0"
__email__ = "jianfeng.sunmt@gmail.com"
__maintainer__ = "Jianfeng Sun"


class BashCLI:

    def __init__(
            self,
            **kwargs,
    ):
        self.kwargs = kwargs
        self.job_fpn = self.kwargs['job_fp'] + self.kwargs['job_fn'] + '.job'

    def shell_slurm_to_many(self, ):
        return "sbatch {}".format(self.job_fpn)